local EpsiIskarAssist = LibStub("AceAddon-3.0"):NewAddon("EpsiIskarAssist", "AceConsole-3.0", "AceEvent-3.0")
local DebuffsDB = nil -- Shortcut

-- Local variable to speedup things
local UnitName = UnitName
local UnitGUID = UnitGUID
local UnitClass = UnitClass
local IsInRaid = IsInRaid
local UnitGroupRolesAssigned = UnitGroupRolesAssigned
local GetNumGroupMembers = GetNumGroupMembers
local UnitAffectingCombat = UnitAffectingCombat


local ipairs = ipairs
local tonumber = tonumber

local ADDON_NAME = "Epsilon Iskar Assist"

-- *******************
-- *** Addon Version
-- *** 0.6.3.1 (Release)
local MajorVersion = 0
local MinorVersion = 6
-- Stage Number
-- 0 : Alpha
-- 1 : Beta
-- 2 : Release candidate
-- 3 : Final release
local StageVersion = 3
local RevisionVersion = 1
local BuildVersion = -1 -- currently not used and undefined

-- *************
-- *** Media ***
-- *************
local SKAM_BAR = [[Interface\Addons\EpsiIskarAssist\Media\skam_bar]]

-- ********************
-- *** Common Data ****
-- ********************
local IskarID = 90316
local IskarEncounterID = 1788

local HealerCount, PreviousHealerCount = 0, 0
local TankCount, PreviousTankCount = 0, 0
local DpsCount, PreviousDpsCount = 0, 0

local Healers = {}
local Tanks = {}
local Dps = {}

local PLAYER_FRAME_WIDTH, PLAYER_FRAME_HEIGHT = 100, 25
local DPS_PLAYER_FRAME_WIDTH, DPS_PLAYER_FRAME_HEIGHT = 75, 25

local needUpdate = false
local needHide = false

local _, MyClass = UnitClass("player")

local Winds = {}
local Wounds = {}
local DarkBindings = {}
local Corruptions = {}
local FelBomb = nil
local EyeOfAnzu = nil

-- *************************
-- *** Common Frame Data ***
-- *************************
local EpsiIskarFrame = nil
local HealerFrames = {}
local TankFrames = {}
local DpsFrames = {}
local EyeOFAnzuFrame = {}

local HealerGroupFrame = nil
local TankGroupFrame = nil
local DpsGroupFrame = nil

-- ****************************
-- *** Localized spell name ***
-- ****************************
local AURA_EYE_OF_ANZU, _, AURA_EYE_OF_ANZU_ICON = GetSpellInfo(179202)
local AURA_PHANTASMAL_WINDS, _, AURA_PHANTASMAL_WINDS_ICON = GetSpellInfo(181957)
local AURA_PHANTASMAL_WOUNDS, _, AURA_PHANTASMAL_WOUNDS_ICON = GetSpellInfo(182325)
local AURA_PHANTASMAL_FEL_BOMB = GetSpellInfo(179219)
local AURA_FEL_BOMB, _, AURA_FEL_BOMB_ICON = GetSpellInfo(181753)
local AURA_PHANTASMAL_CORRUPTION, _, AURA_PHANTASMAL_CORRUPTION_ICON = GetSpellInfo(181824)
local AURA_DARK_BINDINGS, _, AURA_DARK_BINDINGS_ICON = GetSpellInfo(185510)
local AURA_RADIANCE_OF_ANZU, _, AURA_RADIANCE_OF_ANZU_ICON = GetSpellInfo(185239)

-- ******************
-- *** Color Aura ***
-- ******************
local AURA_PHANTASMAL_WINDS_COLOR = {}
local AURA_PHANTASMAL_WOUNDS_COLOR = {}
local AURA_FEL_BOMB_COLOR = {}
local AURA_PHANTASMAL_CORRUPTION_COLOR = {}
local AURA_DARK_BINDINGS_COLOR = {}

-- ********************
-- *** Macro Format ***
-- ********************
--local MacroEyeOfAnzu = "/target %s"
local MacroEyeOfAnzu = "/target %s\n/click ExtraActionButton1\n/targetlasttarget"

-- ***********************************
-- *** Local Misc Frame Functions  ***
-- ***********************************
local function HandleFrameDragStart(frame)
	frame:StartMoving()
end

local function HandleFrameDragStop(frame)
	frame:StopMovingOrSizing()

	x = frame:GetLeft()
	y = frame:GetBottom()

	EpsiIskarAssist.db.profile.xPos = x
	EpsiIskarAssist.db.profile.yPos = y

end

-- ****************************
-- *** Local Misc Functions ***
-- ****************************
local function GetVersionString()
	local stage = ""
	if StageVersion == 0 then
		stage = "Alpha"
	elseif StageVersion == 1 then
		stage = "Beta"
	elseif StageVersion == 2 then
		stage = "Release Candidate"
	elseif StageVersion == 3 then
		stage = "Release"
	end

	local version = string.format("%i.%i.%i.%i (%s)", MajorVersion, MinorVersion, StageVersion, RevisionVersion, stage)

	return version
end


local function GetNpcID(guid)

	if not guid then return -1 end

	local _, _, _, _, _, npcID, _ = strsplit("-", guid)

	return npcID
end

local function GetNpcIDFromUnit(unit)

	if not unit then return -1 end

	local guid = UnitGUID(unit)

	return GetNpcID(guid)
end

local function GetRoleIconText(role, width, height)
	if not height or not width then
		height = 32
		width = 32
	end

	if not role or role == "NONE" then
        return ""
    elseif role == "HEALER" then
        return string.format("|T%s:%i:%i:0:0:256:256:69:130:2:63|t", "Interface/LFGFRAME/UI-LFG-ICON-ROLES", width, height)
    elseif role == "DAMAGER" then
        return string.format("|T%s:%i:%i:0:0:256:256:69:130:69:130|t", "Interface/LFGFRAME/UI-LFG-ICON-ROLES", width, height)
    elseif role == "TANK" then
        return string.format("|T%s:%i:%i:0:0:256:256:3:64:69:130|t", "Interface/LFGFRAME/UI-LFG-ICON-ROLES", width, height)
    end
end

local function GetIconText(icon, width, height)

	if not icon then return "" end

	if not width or not height then
		height = 24
		width = 24

	end

	return string.format("|T%s:%i:%i|t", icon, width, height)

end
local function GetNumMember()
		local num = GetNumGroupMembers()

		difficulty = GetRaidDifficultyID()

		-- Raid Normal
		if difficulty == 14 then
			maxNumber = 30
		-- Raid Heroic
		elseif difficulty == 15 then
			maxNumber = 30
		-- Raid Mythic
		elseif difficulty == 16 then
			maxNumber = 20
		else
			maxNumber = 40
		end

		if num > maxNumber then
			return maxNumber
		else
			return num
		end
end

-- ***********************
-- *** Defaults options ***
-- ************************
local defaults = {
	profile = {
		lock = true,
		show = false,
		scale = 1.0,
		xPos = 100,
		yPos = 500,
		frameStrata = "HIGH",
		font = {
			color = {
				r = 0,
				g = 0,
				b = 0,
			},
			useClassColor = true,
		},
		debuffs = {
			winds = {
				show = true,
				color = {
					r = 1,
					g = 0,
					b = 0,
				},
				priority = 1,
			},
			wounds = {
				show = true,
				color = {
					r = 1,
					g = 0,
					b = 1,
				},
				prority = 4,
			},
			felBomb = {
				show = true,
				color = {
					r = 0,
					g = 1,
					b = 0,
				},
				priority = 0,

			},
			corruption = {
				show = true,
				color = {
					r = 1,
					g = 0.85,
					b = 0,

				},
				priority = 3,
			},
			darkBindings = {
				show = true,
				color = {
					r = 0.49,
					g = 0.29,
					b = 0.09,
				},
				priority = 2,
			}

		},
	eyeOfAnzu = {
		show = true,
		showRadianceOfAnzu = true,
	},
	radianceOfAnzu = {
		show = true,
		showIcon = false,
		stack = {
			low = {
				threshold = 10,
				color = {
					r = 0,
					g = 1,
					b = 0,
				}
			},
			medium = {
				threshold = 20,
				color = {
					r = 1,
					g = 0.5,
					b = 0,
				}
			},
			high = {
				color = {
					r = 1,
					g = 0,
					b = 0,
				}
			}

		}

	}
	}

}

-- *********************
-- *** Load Variable ***
-- *********************
function EpsiIskarAssist:LoadSaveVariables()
	-- Create the shortcut
	DebuffsDB = self.db.profile.debuffs

	local winds = DebuffsDB.winds
	local wounds = DebuffsDB.wounds
	local felBomb = DebuffsDB.felBomb
	local corruption = DebuffsDB.corruption
	local darkBindings = DebuffsDB.darkBindings


	-- Load Color
	AURA_PHANTASMAL_WINDS_COLOR.r, AURA_PHANTASMAL_WINDS_COLOR.g, AURA_PHANTASMAL_WINDS_COLOR.b = winds.color.r, winds.color.g, winds.color.b
	AURA_PHANTASMAL_WOUNDS_COLOR.r, AURA_PHANTASMAL_WOUNDS_COLOR.g, AURA_PHANTASMAL_WOUNDS_COLOR.b = wounds.color.r, wounds.color.g, wounds.color.b
	AURA_FEL_BOMB_COLOR.r, AURA_FEL_BOMB_COLOR.g, AURA_FEL_BOMB_COLOR.b = felBomb.color.r, felBomb.color.g, felBomb.color.b
	AURA_PHANTASMAL_CORRUPTION_COLOR.r, AURA_PHANTASMAL_CORRUPTION_COLOR.g, AURA_PHANTASMAL_CORRUPTION_COLOR.b = corruption.color.r, corruption.color.g, corruption.color.b
	AURA_DARK_BINDINGS_COLOR.r, AURA_DARK_BINDINGS_COLOR.g, AURA_DARK_BINDINGS_COLOR.b = darkBindings.color.r, darkBindings.color.g, darkBindings.color.b

end

-- ***************************
-- *** Save Color Variable ***
-- ***************************
function EpsiIskarAssist:SaveColorVariable(debuff, r, g, b)
	if debuff == FelBomb then
		DebuffsDB.felBomb.color.r, DebuffsDB.felBomb.color.g, DebuffsDB.felBomb.color.b = r, g, b
	elseif debuff == AURA_PHANTASMAL_WINDS then
		DebuffsDB.winds.color.r, DebuffsDB.winds.color.g, DebuffsDB.winds.color.b = r, g, b
	elseif debuff == AURA_PHANTASMAL_WOUNDS then
		DebuffsDB.wounds.color.r, DebuffsDB.wounds.color.g, DebuffsDB.wounds.color.b = r, g, b
	elseif debuff == AURA_PHANTASMAL_CORRUPTION then
		DebuffsDB.corruption.color.r, DebuffsDB.corruption.color.g, DebuffsDB.corruption.color.b = r, g, b
	elseif debuff == AURA_DARK_BINDINGS then
		DebuffsDB.darkBindings.color.r, DebuffsDB.darkBindings.color.g, DebuffsDB.darkBindings.color.b = r, g, b
	end
end

-- ****************************
-- *** Addon Initialization ***
-- ****************************
function EpsiIskarAssist:OnInitialize()
	-- Ace DB
	self.db = LibStub("AceDB-3.0"):New("EpsiIskarAssistDB", defaults)

	-- Load Save Variables
	self:LoadSaveVariables()

	-- Load Frame variables
	local xPos = self.db.profile.xPos
	local yPos = self.db.profile.yPos
	local show = self.db.profile.show
	local lock = self.db.profile.lock


	EpsiIskarFrame = CreateFrame("frame", nil, UIParent)
	EpsiIskarFrame:SetFrameStrata(self.db.profile.frameStrata)
	EpsiIskarFrame:SetWidth(400)
	EpsiIskarFrame:SetHeight(400)
	--EpsiIskarFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 100, 500)
	EpsiIskarFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", xPos, yPos)

	-- COMBAT LOG EVENT
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "HandleCombatLog")

	-- Events used to upate roster
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "HandleRaidChange")
	self:RegisterEvent("GROUP_ROSTER_UPDATE", "HandleRaidChange")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "HandleRaidChange")
	self:RegisterEvent("UNIT_NAME_UPDATE", "HandleRaidChange")

	-- Event used to handle auto-show
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "HandleTargetChanged")
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "HandleMouseoverChanged")

	-- Currently unused [TODO]
	self:RegisterEvent("ENCOUNTER_START", "HandleEncounterStart")
	self:RegisterEvent("ENCOUNTER_END", "HandleEncounterStop")

	-- Event used to delay some actions
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "HandleDelayedAction")

	self:CreateTankCategoryFrame()
	self:CreateHealerCategoryFrame()
	self:CreateDpsCategoryFrame()
	self:CreateEyeOfAnzuFrame()

	self:RegisterChatCommand("iskar", "HandleChatCommands")
	self:RegisterChatCommand("eia", "HandleChatCommands")

	-- Move
	EpsiIskarFrame:EnableMouse(false)
	EpsiIskarFrame:SetMovable(false)

	if lock then
		self:LockFrames()
	else
		self:UnlockFrames()
	end

	if show then
		EpsiIskarFrame:Show()
	else
		EpsiIskarFrame:Hide()
	end

	EpsiIskarFrame:RegisterForDrag("LeftButton")
	EpsiIskarFrame:SetScript("OnDragStart", HandleFrameDragStart)
	EpsiIskarFrame:SetScript("OnDragStop", HandleFrameDragStop)




	-- Create the options
	self:SetupOptions()

end

-- *****************************
-- *** Addon Misc Functions  ***
-- *****************************
function EpsiIskarAssist:ClearHealerData()
	for name, index in pairs(Healers) do
		Healers[name] = nil
	end

	Healers = {}
end

function EpsiIskarAssist:AddHealer(name)
	if Healers[name] then return end

	Healers[name] = -1
end

function EpsiIskarAssist:ClearTankData()
	for name, index in pairs(Tanks) do
		Tanks[name] = nil
	end

	Tanks = {}
end

function EpsiIskarAssist:AddTank(name)
	if Tanks[name] then return end

	Tanks[name] = -1
end

function EpsiIskarAssist:ClearDpsData()
	for name, index in pairs(Dps) do
		Dps[name] = nil
	end

	Dps = {}
end

function EpsiIskarAssist:AddDps(name)
	if Dps[name] then return end

	--print("Add DPS", name)
	Dps[name] = -1
end


function EpsiIskarAssist:AddDebuff(name, debuffName)

	if self:HasDebuff(name, debuffName) then return end

	if debuffName == AURA_PHANTASMAL_WINDS then
		tinsert(Winds, name)
	elseif debuffName == AURA_PHANTASMAL_WOUNDS then
		tinsert(Wounds, name)
	elseif debuffName == AURA_DARK_BINDINGS then
		tinsert(DarkBindings, name)
	elseif debuffName == AURA_PHANTASMAL_CORRUPTION then
		tinsert(Corruptions, name)
	end
end

function EpsiIskarAssist:RemoveDebuff(name, debuffName)

	local hasDebuff, index = self:HasDebuff(name, debuffName)

	if not hasDebuff then return end

	if debuffName == AURA_PHANTASMAL_WINDS then
		tremove(Winds, index)
	elseif debuffName == AURA_PHANTASMAL_WOUNDS then
		tremove(Wounds, index)
	elseif debuffName == AURA_DARK_BINDINGS then
		tremove(DarkBindings, index)
	elseif debuffName == AURA_PHANTASMAL_CORRUPTION then
		tremove(Corruptions, index)
	end
end


function EpsiIskarAssist:HasDebuff(name, debuffName)

	if debuffName == AURA_FEL_BOMB then
		if FelBomb and FelBomb == name then return true, nil end
	elseif debuffName == AURA_PHANTASMAL_WINDS then
		for index, playerName in ipairs(Winds) do
			if name == playerName then return true, index end
		end
	elseif debuffName == AURA_PHANTASMAL_WOUNDS then
		for index, playerName in ipairs(Wounds) do
			if name == playerName then return true, index end
		end
	elseif debuffName == AURA_DARK_BINDINGS then
		for index, playerName in ipairs(DarkBindings) do
			if name == playerName then return true, index end
		end
	elseif debuffName == AURA_PHANTASMAL_CORRUPTION then
		for index, playerName in ipairs(Corruptions) do
			if name == playerName then return true, index end
		end
	end

	return false, nil
end

function EpsiIskarAssist:SetFelBomb(name)

	-- if the user doesn't want show this debuff, don't continue
	if not DebuffsDB.felBomb.show then return end

	if not FelBomb and not name then return end

	local n = nil
	if name then
		n = name
		FelBomb = name
	else
		n = FelBomb
		FelBomb = nil
	end

	local role = UnitGroupRolesAssigned(n)
	local frame = nil

	if role == "HEALER" then
		frame = HealerFrames[Healers[n]]
	elseif role == "TANK" then
		frame = TankFrames[Tanks[n]]
	elseif role == "DAMAGER" then
		frame = DpsFrames[Dps[n]]
	else
		return -- return if any role exist (fix lua error)
	end

	-- If Felbomb is ~= nil, Felbomb is added
	if FelBomb then
		frame.texture:SetVertexColor(AURA_FEL_BOMB_COLOR.r, AURA_FEL_BOMB_COLOR.g, AURA_FEL_BOMB_COLOR.b, 1)
	else
		if self:HasDebuff(n, AURA_PHANTASMAL_WINDS) then
				frame.texture:SetVertexColor(AURA_PHANTASMAL_WINDS_COLOR.r, AURA_PHANTASMAL_WINDS_COLOR.g, AURA_PHANTASMAL_WINDS_COLOR.b, 1)
		elseif self:HasDebuff(n, AURA_DARK_BINDINGS) then
				frame.texture:SetVertexColor(AURA_DARK_BINDINGS_COLOR.r, AURA_DARK_BINDINGS_COLOR.g, AURA_DARK_BINDINGS_COLOR.b, 1)
		elseif self:HasDebuff(n, AURA_PHANTASMAL_CORRUPTION) then
				frame.texture:SetVertexColor(AURA_PHANTASMAL_CORRUPTION.r, AURA_PHANTASMAL_CORRUPTION.g, AURA_PHANTASMAL_CORRUPTION.b, 1)
		elseif self:HasDebuff(n, AURA_PHANTASMAL_WOUNDS) then
				frame.texture:SetVertexColor(AURA_PHANTASMAL_WOUNDS_COLOR.r, AURA_PHANTASMAL_WOUNDS_COLOR.g, AURA_PHANTASMAL_WOUNDS_COLOR.b, 1)
		else
				frame.texture:SetVertexColor(1, 1, 1, 1)
		end
	end

end

function EpsiIskarAssist:SetEyeOfAnzu(name)
	if not name and not EyeOfAnzu then return end

	if name then
		local role = UnitGroupRolesAssigned(name)
		local _, class = UnitClass(name)
		local color = RAID_CLASS_COLORS[class]

		EyeOfAnzuFrame.name:SetText(string.format("|c%s%s|r", color.colorStr, name))
		if role then
			EyeOfAnzuFrame.role:SetText(GetRoleIconText(role, 22, 22))
		end
	else
		EyeOfAnzuFrame.name:SetText("")
		EyeOfAnzuFrame.role:SetText("")
	end

	EyeOfAnzu = name


end

function EpsiIskarAssist:AddWind(name)

	-- if the user doesn't want show this debuff, don't continue
	if not DebuffsDB.winds.show then return end

	local role = UnitGroupRolesAssigned(name)
	local frame = nil
		--self:Print(role)
	if not role or role == "NONE" then return end

		--self:Print("after role")
	if role == "HEALER" then
		frame = HealerFrames[Healers[name]]
	elseif role == "TANK" then
		frame = TankFrames[Tanks[name]]
	elseif role == "DAMAGER" then
		frame = DpsFrames[Dps[name]]
	end


	if not frame then return end

	self:AddDebuff(name, AURA_PHANTASMAL_WINDS)

	-- Color Priority ( FelBomb > Winds > Wound)
	if not self:HasDebuff(name, AURA_FEL_BOMB) then
		--frame.texture:SetVertexColor(1, 0, 0, 1)
		frame.texture:SetVertexColor(AURA_PHANTASMAL_WINDS_COLOR.r, AURA_PHANTASMAL_WINDS_COLOR.g, AURA_PHANTASMAL_WINDS_COLOR.b, 1)
	end
end

function EpsiIskarAssist:RemoveWind(name)

	-- if the user doesn't want show this debuff, don't continue
	if not DebuffsDB.winds.show then return end

	local role = UnitGroupRolesAssigned(name)
	local frame = nil
	if not role or role == "NONE" then return end

	if role == "HEALER" then
		frame = HealerFrames[Healers[name]]
	elseif role == "TANK" then
		frame = TankFrames[Tanks[name]]
	elseif role == "DAMAGER" then
		frame = DpsFrames[Dps[name]]
	end

	if not frame then return end

	self:RemoveDebuff(name, AURA_PHANTASMAL_WINDS)

	-- Color Priority ( FelBomb > Winds > Corruption > Wound)
	if self:HasDebuff(name, AURA_FEL_BOMB) then
		return
	elseif self:HasDebuff(name, AURA_DARK_BINDINGS) then
		frame.texture:SetVertexColor(AURA_DARK_BINDINGS_COLOR.r, AURA_DARK_BINDINGS_COLOR.g, AURA_DARK_BINDINGS_COLOR.b, 1)
	elseif self:HasDebuff(name, AURA_PHANTASMAL_CORRUPTION) then
		frame.texture:SetVertexColor(AURA_PHANTASMAL_CORRUPTION_COLOR.r, AURA_PHANTASMAL_CORRUPTION_COLOR.g, AURA_PHANTASMAL_CORRUPTION_COLOR.b, 1)
	elseif self:HasDebuff(name, AURA_PHANTASMAL_WOUNDS) then
		frame.texture:SetVertexColor(AURA_PHANTASMAL_WOUNDS_COLOR.r, AURA_PHANTASMAL_WOUNDS_COLOR.g, AURA_PHANTASMAL_WOUNDS_COLOR.b, 1)
	else
		frame.texture:SetVertexColor(1, 1, 1, 1)
	end

end

function EpsiIskarAssist:AddWound(name)

	-- if the user doesn't want show this debuff, don't continue
	if not DebuffsDB.wounds.show then return end

	local role = UnitGroupRolesAssigned(name)
	local frame = nil
		--self:Print(role)
	if not role or role == "NONE" then return end

		--self:Print("after role")
	if role == "HEALER" then
		frame = HealerFrames[Healers[name]]
	elseif role == "TANK" then
		frame = TankFrames[Tanks[name]]
	elseif role == "DAMAGER" then
		frame = DpsFrames[Dps[name]]
	end


	if not frame then return end

	self:AddDebuff(name, AURA_PHANTASMAL_WOUNDS)

	-- Color Priority ( FelBomb > Winds > Wound)
	if not self:HasDebuff(name, AURA_FEL_BOMB) and not self:HasDebuff(name, AURA_PHANTASMAL_WINDS) and not self:HasDebuff(name, AURA_DARK_BINDINGS) and not self:HasDebuff(name, AURA_PHANTASMAL_CORRUPTION) then
		--frame.texture:SetVertexColor(1, 0, 1, 1)
		frame.texture:SetVertexColor(AURA_PHANTASMAL_WOUNDS_COLOR.r, AURA_PHANTASMAL_WOUNDS_COLOR.g, AURA_PHANTASMAL_WOUNDS_COLOR.b, 1)
	end
end

function EpsiIskarAssist:RemoveWound(name)

	-- if the user doesn't want show this debuff, don't continue
	if not DebuffsDB.wounds.show then return end

	local role = UnitGroupRolesAssigned(name)
	local frame = nil

	if not role or role == "NONE" then return end

	if role == "HEALER" then
		frame = HealerFrames[Healers[name]]
	elseif role == "TANK" then
		frame = TankFrames[Tanks[name]]
	elseif role == "DAMAGER" then
		frame = DpsFrames[Dps[name]]
	end

	if not frame then return end

	self:RemoveDebuff(name, AURA_PHANTASMAL_WOUNDS)

	-- Color Priority ( FelBomb > Winds > Dark Bindings > Corruption > Wound)
	if self:HasDebuff(name, AURA_FEL_BOMB) then
		return
	elseif self:HasDebuff(name, AURA_PHANTASMAL_WINDS) then
		return
	elseif self:HasDebuff(name, AURA_DARK_BINDINGS) then
		return
	elseif self:HasDebuff(name, AURA_PHANTASMAL_CORRUPTION) then
		return
	else
		frame.texture:SetVertexColor(1, 1, 1, 1)
	end

end


function EpsiIskarAssist:AddDarkBinding(name)

	-- if the user doesn't want show this debuff, don't continue
	if not DebuffsDB.darkBindings.show then return end

	local role = UnitGroupRolesAssigned(name)
	local frame = nil
		--self:Print(role)
	if not role or role == "NONE" then return end

		--self:Print("after role")
	if role == "HEALER" then
		frame = HealerFrames[Healers[name]]
	elseif role == "TANK" then
		frame = TankFrames[Tanks[name]]
	elseif role == "DAMAGER" then
		frame = DpsFrames[Dps[name]]
	end


	if not frame then return end

	self:AddDebuff(name, AURA_DARK_BINDINGS)

	-- Color Priority ( FelBomb > Winds > DarkBindings > Wound)
	if not self:HasDebuff(name, AURA_FEL_BOMB) and not self:HasDebuff(name, AURA_PHANTASMAL_WINDS) then
		--frame.texture:SetVertexColor(1, 0, 1, 1)
		frame.texture:SetVertexColor(AURA_DARK_BINDINGS_COLOR.r, AURA_DARK_BINDINGS_COLOR.g, AURA_DARK_BINDINGS_COLOR.b, 1)
	end
end

function EpsiIskarAssist:RemoveDarkBinding(name)

	-- if the user doesn't want show this debuff, don't continue
	if not DebuffsDB.darkBindings.show then return end

	local role = UnitGroupRolesAssigned(name)
	local frame = nil

	if not role or role == "NONE" then return end

	if role == "HEALER" then
		frame = HealerFrames[Healers[name]]
	elseif role == "TANK" then
		frame = TankFrames[Tanks[name]]
	elseif role == "DAMAGER" then
		frame = DpsFrames[Dps[name]]
	end

	if not frame then return end

	self:RemoveDebuff(name, AURA_DARK_BINDINGS)

	-- Color Priority ( FelBomb > Winds > DarkBindings > Wound)
	if self:HasDebuff(name, AURA_FEL_BOMB) then
		return
	elseif self:HasDebuff(name, AURA_PHANTASMAL_WINDS) then
		return
	elseif self:HasDebuff(name, AURA_PHANTASMAL_CORRUPTION) then
		frame.texture:SetVertexColor(AURA_PHANTASMAL_CORRUPTION_COLOR.r, AURA_PHANTASMAL_CORRUPTION_COLOR.g, AURA_PHANTASMAL_CORRUPTION_COLOR.b, 1)
	elseif self:HasDebuff(name, AURA_PHANTASMAL_WOUNDS) then
		frame.texture:SetVertexColor(AURA_PHANTASMAL_WOUNDS_COLOR.r, AURA_PHANTASMAL_WOUNDS_COLOR.g, AURA_PHANTASMAL_WOUNDS_COLOR.b, 1)
	else
		frame.texture:SetVertexColor(1, 1, 1, 1)
	end

end

function EpsiIskarAssist:AddCorruption(name)

	-- if the user doesn't want show this debuff, don't continue
	if not DebuffsDB.corruption.show then return end

	local role = UnitGroupRolesAssigned(name)
	local frame = nil
		--self:Print(role)
	if not role or role == "NONE" then return end

		--self:Print("after role")
	if role == "HEALER" then
		frame = HealerFrames[Healers[name]]
	elseif role == "TANK" then
		frame = TankFrames[Tanks[name]]
	elseif role == "DAMAGER" then
		frame = DpsFrames[Dps[name]]
	end


	if not frame then return end

	self:AddDebuff(name, AURA_PHANTASMAL_CORRUPTION)

	-- Color Priority ( FelBomb > Winds > DarkBindings > Corruption > Wound)
	if not self:HasDebuff(name, AURA_FEL_BOMB) and not self:HasDebuff(name, AURA_PHANTASMAL_WINDS) and not self:HasDebuff(name, AURA_DARK_BINDINGS) then
		--frame.texture:SetVertexColor(1, 0, 1, 1)
		frame.texture:SetVertexColor(AURA_PHANTASMAL_CORRUPTION_COLOR.r, AURA_PHANTASMAL_CORRUPTION_COLOR.g, AURA_PHANTASMAL_CORRUPTION_COLOR.b, 1)
	end
end

function EpsiIskarAssist:RemoveCorruption(name)

	-- if the user doesn't want show this debuff, don't continue
	if not DebuffsDB.corruption.show then return end

	local role = UnitGroupRolesAssigned(name)
	local frame = nil

	if not role or role == "NONE" then return end

	if role == "HEALER" then
		frame = HealerFrames[Healers[name]]
	elseif role == "TANK" then
		frame = TankFrames[Tanks[name]]
	elseif role == "DAMAGER" then
		frame = DpsFrames[Dps[name]]
	end

	if not frame then return end

	self:RemoveDebuff(name, AURA_PHANTASMAL_CORRUPTION)

	-- Color Priority ( FelBomb > Winds > DarkBindings > Wound)
	if self:HasDebuff(name, AURA_FEL_BOMB) then
		return
	elseif self:HasDebuff(name, AURA_PHANTASMAL_WINDS) then
		return
	elseif self:HasDebuff(name, AURA_DARK_BINDINGS) then
		return
	elseif self:HasDebuff(name, AURA_PHANTASMAL_WOUNDS) then
		frame.texture:SetVertexColor(AURA_PHANTASMAL_WOUNDS_COLOR.r, AURA_PHANTASMAL_WOUNDS_COLOR.g, AURA_PHANTASMAL_WOUNDS_COLOR.b, 1)
	else
		frame.texture:SetVertexColor(1, 1, 1, 1)
	end

end

function EpsiIskarAssist:UpdateRadianceOfAnzu(name)
	if not self.db.profile.radianceOfAnzu.show and not self.db.profile.eyeOfAnzu.showRadianceOfAnzu then return end

	local role = UnitGroupRolesAssigned(name)
	local frame = nil

	if not role or role == "NONE" then return end

	if role == "HEALER" then
		frame = HealerFrames[Healers[name]]
	elseif role == "TANK" then
		frame = TankFrames[Tanks[name]]
	elseif role == "DAMAGER" then
		frame = DpsFrames[Dps[name]]
	end

	if not frame then return end

	-- Line test
	--local _, _, _, stack = UnitDebuff(name, "Grâce salvatrice")
	local _, _, _, stack = UnitDebuff(name, AURA_RADIANCE_OF_ANZU)
	--self:Printf("Update radiance of Anzue on %s with %i stack(s)", name, stack)

	if stack and stack > 0 then
		if self.db.profile.radianceOfAnzu.showIcon then
			frame.radianceOfAnzuIcon:Show()
		end
		-- color
		local color = nil
		if stack < self.db.profile.radianceOfAnzu.stack.low.threshold then
			color = self.db.profile.radianceOfAnzu.stack.low.color
		elseif stack < self.db.profile.radianceOfAnzu.stack.medium.threshold then
			color = self.db.profile.radianceOfAnzu.stack.medium.color
		else
			color = self.db.profile.radianceOfAnzu.stack.high.color
		end

		if self.db.profile.radianceOfAnzu.show then
			frame.radianceOfAnzuStack:SetTextColor(color.r, color.g, color.b)
			frame.radianceOfAnzuStack:Show()
			frame.radianceOfAnzuStack:SetText(stack)
		end

		if self.db.profile.eyeOfAnzu.show and self.db.profile.eyeOfAnzu.showRadianceOfAnzu then
			EyeOFAnzuFrame.stack:Show()
			EyeOFAnzuFrame.stack:SetText(string.format("(%i)", stack))
			EyeOFAnzuFrame.stack:SetTextColor(color.r, color.g, color.b)
		end
	else
			frame.radianceOfAnzuStack:Hide()
			frame.radianceOfAnzuIcon:Hide()
			EyeOFAnzuFrame.stack:Hide()
	end
end

-- **********************
-- *** Event Handlers ***
-- **********************
function EpsiIskarAssist:HandleRaidChange()

	if UnitAffectingCombat("player") then
		needUpdate = true
		return
	end

	local healers = {}
	local tanks = {}
	local dps = {}

	local healerNum, tankNum, dpsNum = 0, 0, 0

	for i = 1, GetNumMember() do
		local unitID = string.format("raid%i", i)
		local name = UnitName(unitID)
		--print(name, role, unitID)
		if name ~= UNKNOWNOBJECT then
			local role = UnitGroupRolesAssigned(unitID)
			if role == "HEALER" then  -- role == "DAMAGER" or "HEALER"
				tinsert(healers, name)
				healerNum = healerNum + 1
			elseif role == "TANK" then
				tinsert(tanks, name)
				tankNum = tankNum + 1
			elseif role == "DAMAGER" then
				tinsert(dps, name)
				dpsNum = dpsNum + 1
			end
		end
	end

	-- TEST
	self:CheckRoleChange(healerNum, HealerCount, PreviousHealerCount, "HEALER", healers)
	self:CheckRoleChange(tankNum, TankCount, PreviousTankCount, "TANK", tanks)
	self:CheckRoleChange(dpsNum, DpsCount, PreviousDpsCount, "DAMAGER", dps)

end

function EpsiIskarAssist:HandleCombatLog(event, timestamp, message, _, sourceGUID, sourceName, _, _, destGUID, destName, destFlags, destFlags2, ...)
	local isPlayer = bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0
     local isFriendly = bit.band(destFlags, COMBATLOG_OBJECT_REACTION_FRIENDLY) > 0
	 local isInRaid = bit.band(destFlags, COMBATLOG_OBJECT_AFFILIATION_RAID + COMBATLOG_OBJECT_AFFILIATION_PARTY + COMBATLOG_OBJECT_AFFILIATION_MINE) > 0

    -- check if it's a friendly player who is in the raid
    if isPlayer and isFriendly and isInRaid then
		local destName, _ = strsplit("-", destName)
		if message == "SPELL_AURA_APPLIED" or message  == "SPELL_AURA_REFRESH" or message == "SPELL_AURA_APPLIED_DOSE" then
			local spellID, spellName = ...

			if spellName == AURA_EYE_OF_ANZU then -- or spellID == 41635 test
				self:SetEyeOfAnzu(destName)
			elseif spellName == AURA_PHANTASMAL_WINDS then -- 121557 / test
				self:AddWind(destName)
			elseif spellName == AURA_FEL_BOMB then --
				self:SetFelBomb(destName)
			elseif spellName == AURA_PHANTASMAL_WOUNDS then
				self:AddWound(destName)
			elseif spellName == AURA_DARK_BINDINGS then -- 17 / Test
				self:AddDarkBinding(destName)
			elseif spellName == AURA_PHANTASMAL_CORRUPTION then
				self:AddCorruption(destName)
			elseif spellName == AURA_RADIANCE_OF_ANZU or spellID == 155274 then
				self:UpdateRadianceOfAnzu(destName)
			end
		elseif message == "SPELL_AURA_REMOVED" or message == "SPELL_AURA_REMOVED_DOSE" then
			local spellID, spellName = ...

			if spellName == AURA_EYE_OF_ANZU then -- or spellID == 41635 test
				self:SetEyeOfAnzu(nil)
			elseif spellName == AURA_PHANTASMAL_WINDS then -- spellID == 139
				self:RemoveWind(destName)
			elseif spellName == AURA_FEL_BOMB then -- or spellID == 111759
				self:SetFelBomb(nil)
			elseif spellName == AURA_PHANTASMAL_WOUNDS then -- spellID == 77489
				self:RemoveWound(destName)
			elseif spellName == AURA_DARK_BINDINGS then
				self:RemoveDarkBinding(destName)
			elseif spellName == AURA_PHANTASMAL_CORRUPTION then
				self:RemoveCorruption(destName)
			elseif spellName == AURA_RADIANCE_OF_ANZU or spellID == 155274 then
				self:UpdateRadianceOfAnzu(destName)
			end
		end

	elseif message == "UNIT_DIED" then
			if tonumber(GetNpcID(destGUID)) == IskarID then
				needHide = true
			end
	end
end

function EpsiIskarAssist:HandleTargetChanged()

	if UnitAffectingCombat("player") then return end

	local npcID = GetNpcIDFromUnit("target")

	if tonumber(npcID) == IskarID and not UnitIsDead("target") then
		EpsiIskarFrame:Show()
	end
end

function EpsiIskarAssist:HandleMouseoverChanged()

	if UnitAffectingCombat("player") then return end

	local npcID = GetNpcIDFromUnit("mouseover")

	if tonumber(npcID) == IskarID and not UnitIsDead("mouseover") then
		EpsiIskarFrame:Show()
	end
end

function EpsiIskarAssist:HandleEncounterStart(event, encounterID, ...)
	--print("Encounter START", arg1)
end

function EpsiIskarAssist:HandleEncounterStop(event, encounterID, ...)
	--print("Encounter Stop", arg1)
end

function EpsiIskarAssist:HandleDelayedAction()
	if needUpdate then
		self:HandleRaidChange()
		needUpdate = false
	elseif needHide then
		EpsiIskarFrame:Hide()
		needHide = false
	end
end

function EpsiIskarAssist:HandleChatCommands(input)
	if input == "lock" then
		self:Print("EpsiIskarAssist is now locked")
		self:LockFrames()
		self.db.profile.lock = true
	elseif input == "unlock" then
		self:Print("EpsiIskarAssist is now unlocked")
		self:UnlockFrames()
		self.db.profile.lock = false
	elseif input == "show" then
		self:Print("EpsiIskarAssist is now shown")
		EpsiIskarFrame:Show()
	elseif input == "hide" then
		self:Print("EpsiIskarAssist is now hidden")
		EpsiIskarFrame:Hide()
	elseif input == "reload" or input == "refresh" then
		EpsiIskarAssist:HandleRaidChange()
	end


end

-- ***********************
-- *** Frame Functions ***
-- ***********************
function EpsiIskarAssist:LockFrames()
		EpsiIskarFrame:EnableMouse(false)
		EpsiIskarFrame:SetMovable(false)
end

function EpsiIskarAssist:UnlockFrames()
		EpsiIskarFrame:EnableMouse(true)
		EpsiIskarFrame:SetMovable(true)
end


function EpsiIskarAssist:CreatePlayerFrame(unitID, categoryName, index)
	local _, class = UnitClass(unitID)
	local color = RAID_CLASS_COLORS[class]

	local frame = CreateFrame("frame")
	frame:SetSize(PLAYER_FRAME_WIDTH, PLAYER_FRAME_HEIGHT)

	local texture = frame:CreateTexture(nil, "artwork")
	texture:SetTexture(SKAM_BAR)

	texture:SetVertexColor(1, 1, 1, 1)
	texture:SetAllPoints()
	frame.texture = texture

	local radianceOfAnzuIcon = frame:CreateTexture(nil, "OVERLAY")
	radianceOfAnzuIcon:SetTexture(AURA_RADIANCE_OF_ANZU_ICON)
	radianceOfAnzuIcon:SetSize(PLAYER_FRAME_HEIGHT, PLAYER_FRAME_HEIGHT)
	radianceOfAnzuIcon:Hide()
	frame.radianceOfAnzuIcon = radianceOfAnzuIcon

	local radianceOfAnzuStack = frame:CreateFontString(nil, "overlay", "GameFontNormal")
	radianceOfAnzuStack:SetText("")
	radianceOfAnzuStack:SetTextColor(1, 0, 0, 1)
	radianceOfAnzuStack:SetAllPoints(radianceOfAnzuIcon)
	radianceOfAnzuStack:Hide()
	frame.radianceOfAnzuStack = radianceOfAnzuStack

	local button = CreateFrame("button", string.format("%s-%s", categoryName, index), frame, "SecureActionButtonTemplate")
	button:SetAttribute("type1", "macro")
	button:SetAttribute("type2", "spell")
	button:RegisterForClicks("AnyUp")
	button:SetAttribute("macrotext", string.format(MacroEyeOfAnzu, unitID))
	button:SetAttribute("unit", unitID)

	button:SetAllPoints()

	-- for heal
	if MyClass == "PRIEST" then
		local spellName = GetSpellInfo(527)
		button:SetAttribute("spell2", spellName)
	elseif MyClass == "PALADIN" then
		local spellName = GetSpellInfo(4987)
		button:SetAttribute("spell2", spellName)
	elseif MyClass == "MONK" then
		local spellName = GetSpellInfo(115450)
		button:SetAttribute("spell2", spellName)
	elseif MyClass == "SHAMAN" then
		local spellName = GetSpellInfo(77130)
		button:SetAttribute("spell2", spellName)
	elseif MyClass == "DRUID" then
		local spellName = GetSpellInfo(88423)
		button:SetAttribute("spell2", spellName)
	end

	frame.button = button

	local name = frame:CreateFontString(nil, "overlay", "GameFontNormal")
	name:SetText(UnitName(unitID))
	name:SetTextColor(color.r, color.g, color.b, 1)
	name:SetAllPoints(frame)

	frame.name = name

	return frame
end

function EpsiIskarAssist:UpdatePlayerFrame(frame, unitID)
	frame:Show()
	frame.button:Show()
	frame.texture:Show()
	frame.name:Show()

	frame.name:SetText(unitID)

	local color = nil

	if self.db.profile.font.useClassColor then
		local _, class, _ = UnitClass(unitID)
		color = RAID_CLASS_COLORS[class]
	else
		color = self.db.profile.font.color
	end


	frame.name:SetTextColor(color.r, color.g, color.b, 1)

	frame.button:SetAttribute("macrotext", string.format(MacroEyeOfAnzu, unitID))
	frame.button:SetAttribute("unit", unitID)


end

function EpsiIskarAssist:CreateTankCategoryFrame()
	TankGroupFrame = CreateFrame("frame")
	TankGroupFrame:SetSize(100, 45)

	local text = TankGroupFrame:CreateFontString(nil, "overlay", "GameFontNormal")
	text:SetText(string.format("|T%s:%i:%i:0:0:256:256:3:64:69:130|t %s", "Interface/LFGFRAME/UI-LFG-ICON-ROLES", 32, 32, "Tanks"))
	text:SetPoint("TOP")

	TankGroupFrame:SetParent(EpsiIskarFrame)
	TankGroupFrame:SetPoint("TOPLEFT", EpsiIskarFrame, "TOPLEFT", 0, 0)

	TankGroupFrame:Show()

end

function EpsiIskarAssist:CreateHealerCategoryFrame()
	HealerGroupFrame = CreateFrame("frame")
	HealerGroupFrame:SetSize(100, 45)

	local text = HealerGroupFrame:CreateFontString(nil, "overlay", "GameFontNormal")
	text:SetText(string.format("|T%s:%i:%i:0:0:256:256:69:130:2:63|t %s", "Interface/LFGFRAME/UI-LFG-ICON-ROLES", 32, 32, "Healers"))
	text:SetPoint("TOP")
	HealerGroupFrame.text = text

	HealerGroupFrame:SetParent(EpsiIskarFrame)
	HealerGroupFrame:SetPoint("TOP", TankGroupFrame, "BOTTOM", 0, -5)

	HealerGroupFrame:Show()
end

function EpsiIskarAssist:CreateDpsCategoryFrame()
	DpsGroupFrame = CreateFrame("frame")
	DpsGroupFrame:SetSize(150, 45)

	local text = DpsGroupFrame:CreateFontString(nil, "overlay", "GameFontNormal")
	text:SetText(string.format("%s %s", GetRoleIconText("DAMAGER", 32, 32), "Dps"))
	text:SetPoint("TOP")
	DpsGroupFrame.text = text

	DpsGroupFrame:SetParent(EpsiIskarFrame)
	DpsGroupFrame:SetPoint("TOPLEFT", TankGroupFrame, "TOPRIGHT", 5, 0)
	--DpsGroupFrame:SetPoint("TOP", EpsiIskarFrame, "TOP")

	DpsGroupFrame:Show()

end

function EpsiIskarAssist:CreateEyeOfAnzuFrame()

	-- Load Save variable
	local show = self.db.profile.eyeOfAnzu.show

	EyeOfAnzuFrame = CreateFrame("frame")
	EyeOfAnzuFrame:SetSize(200, 50)

	local texture = EyeOfAnzuFrame:CreateTexture(nil, "OVERLAY")
	texture:SetTexture(AURA_EYE_OF_ANZU_ICON)
	texture:SetSize(24, 24)
	texture:SetPoint("LEFT", EyeOfAnzuFrame, "LEFT", 10, 0)

	EyeOfAnzuFrame.texture = texture

	role = EyeOfAnzuFrame:CreateFontString(nil, "overlay", "GameFontNormal")
	role:SetText("")
	role:SetPoint("LEFT", texture, "RIGHT", 10, 0)

	EyeOfAnzuFrame.role = role

	local name = EyeOfAnzuFrame:CreateFontString(nil, "overlay", "GameFontNormal")
	name:SetText("")
	name:SetPoint("LEFT", role, "RIGHT", 2, 0)

	EyeOfAnzuFrame.name = name

	local stack = EyeOfAnzuFrame:CreateFontString(nil, "overlay", "GameFontNormal")
	stack:SetText("")
	stack:SetPoint("LEFT", name, "RIGHT", 2, 0)

	EyeOFAnzuFrame.stack = stack

	EyeOfAnzuFrame:SetParent(EpsiIskarFrame)
	EyeOfAnzuFrame:SetPoint("BOTTOM", EpsiIskarFrame, "TOP", -50, 5)

	if show then
		EyeOfAnzuFrame:Show()
	else
		EyeOfAnzuFrame:Hide()
	end



end



function EpsiIskarAssist:CheckRoleChange(roleNum , currentCount, previousCount, role, roleTable)
		local updateFrameHandler = nil
		local clearHandler = nil
		local addHandler = nil

		if role == "HEALER" then
				clearHandler = function() self:ClearHealerData() end
				updateFrameHandler = function() self:UpdateHealerFrames() end
				addHandler = function(name) self:AddHealer(name) end
		elseif role == "TANK" then
				clearHandler = function() self:ClearTankData() end
				updateFrameHandler = function() self:UpdateTankFrames() end
				addHandler = function(name) self:AddTank(name) end
		elseif role == "DAMAGER" then
				clearHandler = function() self:ClearDpsData() end
				updateFrameHandler = function() self:UpdateDpsFrames() end
				addHandler = function(name) self:AddDps(name) end
		end

		--self:Print(string.format("[Debug] %s Count Changed ! Start the %s update", role, role))

		if roleNum < currentCount then
			 clearHandler()
		end

		for _, name in ipairs(roleTable) do
			addHandler(name)

		end

		if role == "HEALER" then
			PreviousHealerCount = HealerCount
			HealerCount = roleNum
		elseif role == "TANK" then
			PreviousTankCount = TankCount
			TankCount = roleNum
		elseif role == "DAMAGER" then
			PreviousDpsCount = DpsCount
			DpsCount = roleNum
		end

		updateFrameHandler()
end

function EpsiIskarAssist:UpdateHealerFrames()
	self:UpdateRoleFrame("HEALER")
end

function EpsiIskarAssist:UpdateTankFrames()
	self:UpdateRoleFrame("TANK")
end

function EpsiIskarAssist:UpdateDpsFrames()
	self:UpdateRoleFrame("DAMAGER")
end

function EpsiIskarAssist:UpdateAllFrames()
	self:UpdateHealerFrames()
	self:UpdateTankFrames()
	self:UpdateDpsFrames()
end

function EpsiIskarAssist:UpdateRoleFrame(role)
	local previousRoleCount, currentRoleCount = 0,0
	local roleTable = nil
	local roleTableFrame = nil
	local roleParent = nil

	if role == "HEALER" then
				currentRoleCount = HealerCount
				previousRoleCount = PreviousHealerCount
				roleTable = Healers
				roleTableFrame = HealerFrames
				roleParent = HealerGroupFrame
	elseif role == "TANK" then
				currentRoleCount = TankCount
				previousRoleCount = PreviousTankCount
				roleTable = Tanks
				roleTableFrame = TankFrames
				roleParent = TankGroupFrame
	elseif role == "DAMAGER" then
				currentRoleCount = DpsCount
				previousRoleCount = PreviousDpsCount
				roleTable = Dps
				roleTableFrame = DpsFrames
				roleParent = DpsGroupFrame
	end

	if previousRoleCount > currentRoleCount then
		local i = 0

		for name, indexFrame in pairs(roleTable) do
			roleTable[name] = i

			local frame = roleTableFrame[i]

			-- Check
			self:UpdatePlayerFrame(frame, name)

			i = i + 1
		end

		local hiddenFrameCount = previousRoleCount - currentRoleCount
		for y = 0, hiddenFrameCount - 1 do
			local frame = roleTableFrame[currentRoleCount + y]
			frame:Hide()
			frame.button:Hide()
			frame.texture:Hide()
			frame.name:Hide()
			frame.radianceOfAnzuIcon:Hide()
			frame.radianceOfAnzuStack:Hide()

		end
	else
		local i = 0
		for name, indexFrame in pairs(roleTable) do
			if indexFrame < 0 then
				local frame = nil

				if not roleTableFrame[previousRoleCount + i] then
					frame = self:CreatePlayerFrame(name, role, previousRoleCount + i)
					frame:SetParent(roleParent)

					if role == "DAMAGER" then
						local yRatio = math.floor((previousRoleCount + i) / 2)
						if ((previousRoleCount + i) % 2) == 0 then
							frame:SetPoint("TOP", roleParent, "TOP", 0,  -30 - yRatio * DPS_PLAYER_FRAME_HEIGHT)
							frame.radianceOfAnzuIcon:SetPoint("RIGHT", frame, "LEFT", 0, 0)
						else
							frame:SetPoint("TOP", roleParent, "TOP", DPS_PLAYER_FRAME_WIDTH + 5, -30 - yRatio * DPS_PLAYER_FRAME_HEIGHT)
							frame.radianceOfAnzuIcon:SetPoint("LEFT", frame, "RIGHT", 0, 0)
						end

						frame:SetSize(DPS_PLAYER_FRAME_WIDTH, DPS_PLAYER_FRAME_HEIGHT)
					else
						frame.radianceOfAnzuIcon:SetPoint("RIGHT", frame, "LEFT", 0, 0)
						frame:SetPoint("TOP", roleParent, "TOP", 0, -30 - (previousRoleCount + i) * PLAYER_FRAME_HEIGHT)
					end

					roleTableFrame[previousRoleCount + i] = frame

				else
					frame = roleTableFrame[previousRoleCount + i]
				end


				self:UpdatePlayerFrame(frame, name)

				roleTable[name] = previousRoleCount + i

				i = i + 1
			end
		end
	end
	if role == "DAMAGER" then
		roleParent:SetSize(150, 45 + PLAYER_FRAME_HEIGHT * currentRoleCount)
	else
		roleParent:SetSize(100, 45 + PLAYER_FRAME_HEIGHT * currentRoleCount)
	end
end

-- ***************
-- *** Options ***
-- ***************
local AURA_PHANTASMAL_CORRUPTION, _, AURA_PHANTASMAL_CORRUPTION_ICON = GetSpellInfo(181824)
function EpsiIskarAssist:GenerateOptions()
	EpsiIskarAssist.options = {
		type = "group",
		name = string.format("Epsi Iskar Assist v%s", GetVersionString()),
		childGroups = "tab",
		args = {
			general = {
				type = "group",
				name = "General",
				order = 1,
				args = {
					lock = {
						type = "toggle",
						name = "Lock",
						order = 1,
						get = function() return self.db.profile.lock end,
						set = function(_, value)
							if value then
								self:LockFrames()
							else
								self:UnlockFrames()
							end
							self.db.profile.lock = value
						end
					},
					show = {
						type = "toggle",
						name = "Show",
						order = 2,
						get = function() return self.db.profile.show end,
						set = function(_, value)
							if value then
								EpsiIskarFrame:Show()
							else
								EpsiIskarFrame:Hide()
							end

							self.db.profile.show = value
						end,
					},
					scale = {
						type = "range",
						name = "Scale",
						desc = "Currently not implemented",
						order = 3,
						disabled = true,

					},
					frameStrata = {
						type = "select",
						name = "Frame strata",
						desc = [[ FrameStrata is a frame property that determines the coarse order in which UI frames are composited onto the screen.
						The following stratas are ordered in ascending order (a frame in the HIGH Strata will always be coposited on top of a frame in
						the MEDIUM strata) :
						1. BACKGROUND (the lowest stata)
						2. LOW
						3. MEDIUM
						4. HIGH
						5. DIALOG
						6. FULLSCREEN
						7. FULLSCREEN_DIALOG
						8. TOOLTIP (the highest stata) ]],
						values = {['BACKGROUND'] = 'BACKGROUND', ['LOW'] = 'LOW', ['MEDIUM'] = 'MEDIUM',
											['HIGH'] = 'HIGH', ['DIALOG'] = 'DIALOG', ['FULLSCREEN'] = 'FULLSCREEN',
											['FULLSCREEN_DIALOG'] = 'FULLSCREEN_DIALOG', ['TOOLTIP'] = 'TOOLTIP'},
						order = 4,
						get = function() return self.db.profile.frameStrata end,
						set = function(_, value) self.db.profile.frameStrata = value; EpsiIskarFrame:SetFrameStrata(value) end,
					},
					font = {
						type = "group",
						name = "Font",
						inline = true,
						args = {
							color = {
								type = "color",
								name = "Color",
								desc = "Set the name font color.",
								order = 1,
								get = function()
									local color = self.db.profile.font.color
									return color.r, color.g, color.b
								end,
								set = function(_, r, g, b, a)
									local color = self.db.profile.font.color
									color.r, color.g, color.b = r, g, b
									self:UpdateAllFrames()
								end,
								disabled = function() return self.db.profile.font.useClassColor end,



							},
							classColor = {
								type = "toggle",
								name = "Use class color",
								desc = "If set, the name will be colored by the class color.",
								order = 2,
								get = function() return self.db.profile.font.useClassColor end,
								set = function(_, value)
									self.db.profile.font.useClassColor = value
									self:UpdateAllFrames()
								end,

							}

						}

					},

					eyeOfAnzu = {
						type = "group",
						name = string.format("%s %s", GetIconText(AURA_EYE_OF_ANZU_ICON, 18, 18), AURA_EYE_OF_ANZU),
						inline = true,
						args ={
							show = {
								type = "toggle",
								name = "Show",
								desc = "Show the eye of Anzu frame which displays the holder.",
								order = 1,
								get = function() return self.db.profile.eyeOfAnzu.show end,
								set = function(_, value)
									if value then
										EyeOfAnzuFrame:Show()
									else
										EyeOfAnzuFrame:Hide()
									end

									self.db.profile.eyeOfAnzu.show = value
								end,

							},
							showRadianceOfAnzu = {
								type = "toggle",
								name = "Show radiance of Anzu",
								desc = "Display the radiance of Anzu stack of the holder.",
								order = 2,
								get = function() return self.db.profile.eyeOfAnzu.showRadianceOfAnzu end,
								set = function(_, value) self.db.profile.eyeOfAnzu.showRadianceOfAnzu = value end,

							}

						}


					},

					radianceOfAnzu = {
						type = "group",
						name = string.format("%s %s", GetIconText(AURA_RADIANCE_OF_ANZU_ICON, 18, 18), AURA_RADIANCE_OF_ANZU),
						inline = true,
						args = {
							show = {
								type = "toggle",
								name = "Show",
								desc = "Tell if the radiance of Anzu must be shown next to the player frames.",
								order = 1,
								get = function() return self.db.profile.radianceOfAnzu.show end,
								set = function(_, value) self.db.profile.radianceOfAnzu.show = value end,

							},

							showIcon = {
								type = "toggle",
								name = "Show icon",
								desc = "Show the radiance of Anzu icon. If not toggled, only the text is displayed.",
								order = 2,
								get = function() return self.db.profile.radianceOfAnzu.showIcon end,
								set = function(_, value) self.db.profile.radianceOfAnzu.showIcon = value end,
								disabled = function() return not self.db.profile.radianceOfAnzu.show end,

							},
							font = {
								type = "group",
								name = "Font Color Stacks",
								disabled = function() return not self.db.profile.radianceOfAnzu.show end,
								args = {
									colors = {
										type = "group",
										name = "",
										inline = true,
										args = {
											lowColor = {
												type = "color",
												name = "Low",
												desc = "The color used when the radiance of Anzu stack is low.",
												order = 1,
												get = function()
													local color = self.db.profile.radianceOfAnzu.stack.low.color
													return color.r, color.g, color.b
												end,
												set = function(_, r, g, b, a)
													local color = self.db.profile.radianceOfAnzu.stack.low.color
													color.r, color.g, color.b = r, g, b
												end,
											},
											mediumColor = {
												type = "color",
												name = "Medium",
												desc = "The color used when the radiance Of Anzu stack is medium",
												order = 2,
												get = function()
													local color = self.db.profile.radianceOfAnzu.stack.medium.color
													return color.r, color.g, color.b
												end,
												set = function(_, r, g, b, a)
													local color = self.db.profile.radianceOfAnzu.stack.medium.color
													color.r, color.g, color.b = r, g, b
												end,
											},
											HighColor = {
												type = "color",
												name = "High",
												desc = "The color used when the radiance of Anzu stack is neither low nor medium.",
												order = 3,
												get = function()
													local color = self.db.profile.radianceOfAnzu.stack.high.color
													return color.r, color.g, color.b
												end,
												set = function(_, r, g, b, a)
													local color = self.db.profile.radianceOfAnzu.stack.high.color
													color.r, color.g, color.b = r, g, b
												end,
											},
										}

									},
									thresholds = {
										type = "group",
										name = "",
										inline = true,
										args = {
											lowThreshold = {
												type = "range",
												name = "Low Threshold",
												desc = "The value below which a radiance of Anzu stack is considered low.",
												min = 0,
												max = 50,
												step = 1,
												get = function() return self.db.profile.radianceOfAnzu.stack.low.threshold end,
												set = function(_, value) self.db.profile.radianceOfAnzu.stack.low.threshold = value end,

											},
											mediumThreshold = {
												type = "range",
												name = "Medium Threshold",
												desc = "The value below which a radiance of Anzu stack is considered medium.",
												min  = 0,
												max = 50,
												step = 1,
												get = function() return self.db.profile.radianceOfAnzu.stack.medium.threshold end,
												set = function(_, value) self.db.profile.radianceOfAnzu.stack.medium.threshold = value end,
											}

										}

									}


								}

							}

						}

					}
				}

			},

			debuffs = {
				type = "group",
				name = "Debuffs",
				order = 2,
				args = {
					header = {
						type = "description",
						name = string.format("The debuff priority is :\n %s > %s > %s (mythic) > %s > %s",
											AURA_FEL_BOMB, AURA_PHANTASMAL_WINDS, AURA_DARK_BINDINGS, AURA_PHANTASMAL_WOUNDS, AURA_PHANTASMAL_CORRUPTION),
						order = 0,

					},
					winds = {
						type = "group",
						name = string.format("%s %s", GetIconText(AURA_PHANTASMAL_WINDS_ICON, 18, 18), AURA_PHANTASMAL_WINDS),
						inline = true,
						order = 1,
						args = {
							show = {
								type = "toggle",
								name = "Show debuff",
								order = 1,
								get = function() return DebuffsDB.winds.show end,
								set = function(_, value) DebuffsDB.winds.show = value end,

							},
							color = {
								type = "color",
								name = "Debuff color",
								order = 2,
								get = function() return AURA_PHANTASMAL_WINDS_COLOR.r, AURA_PHANTASMAL_WINDS_COLOR.g, AURA_PHANTASMAL_WINDS_COLOR.b end,
								set = function(_, r, g, b, a)
									AURA_PHANTASMAL_WINDS_COLOR.r, AURA_PHANTASMAL_WINDS_COLOR.g, AURA_PHANTASMAL_WINDS_COLOR.b = r, g, b
									self:SaveColorVariable(AURA_PHANTASMAL_WINDS, r, g, b)
								end,

							},
						}
					},

					wounds = {
						type = "group",
						name = string.format("%s %s", GetIconText(AURA_PHANTASMAL_WOUNDS_ICON, 18, 18), AURA_PHANTASMAL_WOUNDS),
						inline = true,
						order = 2,
						args = {
							show = {
								type = "toggle",
								name = "Show debuff",
								order = 1,
								get = function() return DebuffsDB.wounds.show end,
								set = function(_, value) DebuffsDB.wounds.show = value end,

							},
							color = {
								type = "color",
								name = "Debuff color",
								order = 2,
								get = function() return AURA_PHANTASMAL_WOUNDS_COLOR.r, AURA_PHANTASMAL_WOUNDS_COLOR.g, AURA_PHANTASMAL_WOUNDS_COLOR.b end,
								set = function(_, r, g, b, a)
									AURA_PHANTASMAL_WOUNDS_COLOR.r, AURA_PHANTASMAL_WOUNDS_COLOR.g, AURA_PHANTASMAL_WOUNDS_COLOR.b = r, g, b
									self:SaveColorVariable(AURA_PHANTASMAL_WOUNDS, r, g, b)
								end

							},

						}
					},
					felBomb = {
						type = "group",
						name = string.format("%s %s", GetIconText(AURA_FEL_BOMB_ICON, 18, 18), AURA_FEL_BOMB),
						inline = true,
						order = 3,
						args = {
							show = {
								type = "toggle",
								name = "Show debuff",
								order = 1,
								get = function() return DebuffsDB.felBomb.show end,
								set = function(_, value) DebuffsDB.felBomb.show = value end,

							},
							color = {
								type = "color",
								name = "Debuff color",
								order = 2,
								get = function() return AURA_FEL_BOMB_COLOR.r, AURA_FEL_BOMB_COLOR.g, AURA_FEL_BOMB_COLOR.b end,
								set = function(_, r, g, b, a)
									AURA_FEL_BOMB_COLOR.r, AURA_FEL_BOMB_COLOR.g, AURA_FEL_BOMB_COLOR.b = r, g, b
									self:SaveColorVariable(AURA_FEL_BOMB, r, g, b)
								end,



							},
						}
					},
					corruption = {
						type = "group",
						name = string.format("%s%s", GetIconText(AURA_PHANTASMAL_CORRUPTION_ICON, 18, 18), AURA_PHANTASMAL_CORRUPTION),
						inline = true,
						order = 4,
						args = {
							show = {
								type = "toggle",
								name = "Show debuff",
								order = 1,
								get = function() return DebuffsDB.corruption.show end,
								set = function(_, value) DebuffsDB.corruption.show = value end,
							},
							color = {
								type = "color",
								name = "Debuff color",
								order = 2,
								get = function() return AURA_PHANTASMAL_CORRUPTION_COLOR.r, AURA_PHANTASMAL_CORRUPTION_COLOR.g, AURA_PHANTASMAL_CORRUPTION_COLOR.b end,
								set = function(_, r, g, b, a)
									AURA_PHANTASMAL_CORRUPTION_COLOR.r, AURA_PHANTASMAL_CORRUPTION_COLOR.g, AURA_PHANTASMAL_CORRUPTION_COLOR.b = r, g, b
									self:SaveColorVariable(AURA_PHANTASMAL_CORRUPTION, r, g, b)
								end,
							},
						}
					},
					darkBindings = {
						type = "group",
						name = string.format("%s%s", GetIconText(AURA_DARK_BINDINGS_ICON, 18, 18), AURA_DARK_BINDINGS),
						inline = true,
						order = 5,
						args = {
							show = {
								type = "toggle",
								name = "Show debuff",
								order = 1,
								get = function() return DebuffsDB.darkBindings.show end,
								set = function(_, value) DebuffsDB.darkBindings.show = value end,
							},
							color = {
								type = "color",
								name = "Debuff color",
								order = 2,
								get = function() return AURA_DARK_BINDINGS_COLOR.r, AURA_DARK_BINDINGS_COLOR.g, AURA_DARK_BINDINGS_COLOR.b end,
								set = function(_, r, g, b, a)
									AURA_DARK_BINDINGS_COLOR.r, AURA_DARK_BINDINGS_COLOR.g, AURA_DARK_BINDINGS_COLOR.b = r, g, b
									self:SaveColorVariable(AURA_DARK_BINDINGS, r, g, b)
									end,

							},
						}
					}
				}
			}
		}
	}
end

function EpsiIskarAssist:GetOptions()
	if not EpsiIskarAssist.options then
		self:GenerateOptions()
	end
	return EpsiIskarAssist.options
end

function EpsiIskarAssist:SetupOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("EpsiIskarAssist", self:GetOptions())
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("EpsiIskarAssist", "EpsiIskarAssist")
end
